=== modified file 'bzrlib/index.py'
--- bzrlib/index.py	2007-07-15 07:31:37 +0000
+++ bzrlib/index.py	2007-07-15 15:40:37 +0000
@@ -64,7 +64,7 @@
         self.reference_lists = reference_lists
         self._nodes = {}
 
-    def add_node(self, key, references, value):
+    def add_node(self, key, value, references=()):
         """Add a node to the index.
 
         :param key: The key. keys must be whitespace-free utf8.
@@ -200,9 +200,11 @@
     def iter_all_entries(self):
         """Iterate over all keys within the index.
 
-        :return: An iterable of (key, reference_lists, value). There is no
-            defined order for the result iteration - it will be in the most
-            efficient order for the index.
+        :return: An iterable of (key, value) or (key, value, reference_lists).
+            The former tuple is used when there are no reference lists in the
+            index, making the API compatible with simple key:value index types.
+            There is no defined order for the result iteration - it will be in
+            the most efficient order for the index.
         """
         stream = self._transport.get(self._name)
         self._read_prefix(stream)
@@ -232,10 +234,9 @@
                 node_refs = []
                 for ref_list in references:
                     node_refs.append(tuple([self.keys_by_offset[ref][0] for ref in ref_list]))
-                node_refs = tuple(node_refs)
+                yield (key, value, tuple(node_refs))
             else:
-                node_refs = ()
-            yield (key, node_refs, value)
+                yield (key, value)
         if trailers != 1:
             # there must be one line - the empty trailer line.
             raise errors.BadIndexData(self)
@@ -256,9 +257,9 @@
         """Iterate over keys within the index.
 
         :param keys: An iterable providing the keys to be retrieved.
-        :return: An iterable of (key, reference_lists, value). There is no
-            defined order for the result iteration - it will be in the most
-            efficient order for the index.
+        :return: An iterable as per iter_all_entries, but restricted to the
+            keys supplied. No additional keys will be returned, and every
+            key supplied that is in the index will be returned.
         """
         keys = set(keys)
         if not keys:
@@ -364,8 +365,8 @@
 
         :param nodes: An iterable of (key, node_refs, value) entries to add.
         """
-        for (key, node_refs, value) in nodes:
-            self.add_node(key, node_refs, value)
+        for (key, value, node_refs) in nodes:
+            self.add_node(key, value, node_refs)
 
     def iter_all_entries(self):
         """Iterate over all keys within the index
@@ -374,9 +375,14 @@
             defined order for the result iteration - it will be in the most
             efficient order for the index (in this case dictionary hash order).
         """
-        for key, (absent, references, value) in self._nodes.iteritems():
-            if not absent:
-                yield key, references, value
+        if self.reference_lists:
+            for key, (absent, references, value) in self._nodes.iteritems():
+                if not absent:
+                    yield key, value, references
+        else:
+            for key, (absent, references, value) in self._nodes.iteritems():
+                if not absent:
+                    yield key, value
 
     def iter_entries(self, keys):
         """Iterate over keys within the index.
@@ -387,10 +393,16 @@
             efficient order for the index (keys iteration order in this case).
         """
         keys = set(keys)
-        for key in keys.intersection(self._nodes):
-            node = self._nodes[key]
-            if not node[0]:
-                yield key, node[1], node[2]
+        if self.reference_lists:
+            for key in keys.intersection(self._nodes):
+                node = self._nodes[key]
+                if not node[0]:
+                    yield key, node[2], node[1]
+        else:
+            for key in keys.intersection(self._nodes):
+                node = self._nodes[key]
+                if not node[0]:
+                    yield key, node[2]
 
     def validate(self):
         """In memory index's have no known corruption at the moment."""

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2007-07-15 08:30:05 +0000
+++ bzrlib/knit.py	2007-07-15 15:52:54 +0000
@@ -1327,7 +1327,7 @@
         :param deltas: Allow delta-compressed records.
         :param add_callback: If not None, allow additions to the index and call
             this callback with a list of added GraphIndex nodes:
-            [(node, node_refs, value), ...]
+            [(node, value, node_refs), ...]
         """
         self._graph_index = graph_index
         self._deltas = deltas
@@ -1355,7 +1355,7 @@
             this_iteration = pending
             new_nodes = self._get_entries(this_iteration)
             pending = set()
-            for (key, node_refs, value) in new_nodes:
+            for (key, value, node_refs) in new_nodes:
                 # dont ask for ghosties - otherwise
                 # we we can end up looping with pending
                 # being entirely ghosted.
@@ -1384,7 +1384,7 @@
             this_iteration = pending
             new_nodes = self._get_entries(this_iteration)
             pending = set()
-            for (key, node_refs, value) in new_nodes:
+            for (key, value, node_refs) in new_nodes:
                 graph[key] = node_refs[0]
                 # queue parents 
                 pending.update(graph[key])
@@ -1398,7 +1398,7 @@
 
     def get_graph(self):
         """Return a list of the node:parents lists from this knit index."""
-        return [(key, refs[0]) for (key, refs, value) in 
+        return [(key, refs[0]) for (key, value, refs) in 
             self._graph_index.iter_all_entries()]
 
     def num_versions(self):
@@ -1416,14 +1416,14 @@
 
     def get_position(self, version_id):
         """Return data position and size of specified version."""
-        bits = self._get_node(version_id)[2][1:].split(' ')
+        bits = self._get_node(version_id)[1][1:].split(' ')
         return int(bits[0]), int(bits[1])
 
     def get_method(self, version_id):
         """Return compression method of specified version."""
         if not self._deltas:
             return 'fulltext'
-        return self._parent_compression(self._get_node(version_id)[1][1])
+        return self._parent_compression(self._get_node(version_id)[2][1])
 
     def _parent_compression(self, reference_list):
         # use the second reference list to decide if this is delta'd or not.
@@ -1444,8 +1444,8 @@
         if not self._deltas:
             options = ['fulltext']
         else:
-            options = [self._parent_compression(node[1][1])]
-        if node[2][0] == 'N':
+            options = [self._parent_compression(node[2][1])]
+        if node[1][0] == 'N':
             options.append('no-eol')
         return ','.join(options)
 
@@ -1457,7 +1457,7 @@
 
     def get_parents_with_ghosts(self, version_id):
         """Return parents of specified version with ghosts."""
-        return self._get_node(version_id)[1][0]
+        return self._get_node(version_id)[2][0]
 
     def check_versions_present(self, version_ids):
         """Check that all specified versions are present."""
@@ -1481,8 +1481,6 @@
 
         :param versions: a list of tuples:
                          (version_id, options, pos, size, parents).
-        :return: A list of (key, node_refs, value) tuples for insertion
-            into a GraphIndex.
         """
         if not self._add_callback:
             raise errors.ReadOnlyError(self)
@@ -1506,15 +1504,15 @@
                 if 'line-delta' in options:
                     raise KnitCorrupt(self, "attempt to add line-delta in non-delta knit")
                 node_refs = (tuple(parents), )
-            keys[version_id] = (node_refs, value)
+            keys[version_id] = (value, node_refs)
         present_nodes = self._get_entries(keys)
-        for (key, node_refs, value) in present_nodes:
-            if (node_refs, value) != keys[key]:
+        for (key, value, node_refs) in present_nodes:
+            if (value, node_refs) != keys[key]:
                 raise KnitCorrupt(self, "inconsistent details in add_versions")
             del keys[key]
         result = []
-        for key, (node_refs, value) in keys.iteritems():
-            result.append((key, node_refs, value))
+        for key, (value, node_refs) in keys.iteritems():
+            result.append((key, value, node_refs))
         self._add_callback(result)
         
 

=== modified file 'bzrlib/tests/test_index.py'
--- bzrlib/tests/test_index.py	2007-07-15 04:45:19 +0000
+++ bzrlib/tests/test_index.py	2007-07-15 15:40:37 +0000
@@ -41,9 +41,17 @@
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=2\n\n", contents)
 
-    def test_build_index_one_node(self):
-        builder = GraphIndexBuilder()
-        builder.add_node('akey', (), 'data')
+    def test_build_index_one_node_no_refs(self):
+        builder = GraphIndexBuilder()
+        builder.add_node('akey', 'data')
+        stream = builder.finish()
+        contents = stream.read()
+        self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=0\n"
+            "akey\x00\x00\x00data\n\n", contents)
+
+    def test_build_index_one_node_no_refs_accepts_empty_reflist(self):
+        builder = GraphIndexBuilder()
+        builder.add_node('akey', 'data', ())
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=0\n"
@@ -51,7 +59,7 @@
 
     def test_add_node_empty_value(self):
         builder = GraphIndexBuilder()
-        builder.add_node('akey', (), '')
+        builder.add_node('akey', '')
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=0\n"
@@ -63,9 +71,9 @@
         # use three to have a good chance of glitching dictionary hash
         # lookups etc. Insert in randomish order that is not correct
         # and not the reverse of the correct order.
-        builder.add_node('2002', (), 'data')
-        builder.add_node('2000', (), 'data')
-        builder.add_node('2001', (), 'data')
+        builder.add_node('2002', 'data')
+        builder.add_node('2000', 'data')
+        builder.add_node('2001', 'data')
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=0\n"
@@ -76,7 +84,7 @@
 
     def test_build_index_reference_lists_are_included_one(self):
         builder = GraphIndexBuilder(reference_lists=1)
-        builder.add_node('key', ([], ), 'data')
+        builder.add_node('key', 'data', ([], ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=1\n"
@@ -85,7 +93,7 @@
 
     def test_build_index_reference_lists_are_included_two(self):
         builder = GraphIndexBuilder(reference_lists=2)
-        builder.add_node('key', ([], []), 'data')
+        builder.add_node('key', 'data', ([], []))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=2\n"
@@ -94,8 +102,8 @@
 
     def test_node_references_are_byte_offsets(self):
         builder = GraphIndexBuilder(reference_lists=1)
-        builder.add_node('reference', ([], ), 'data')
-        builder.add_node('key', (['reference'], ), 'data')
+        builder.add_node('reference', 'data', ([], ))
+        builder.add_node('key', 'data', (['reference'], ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=1\n"
@@ -105,9 +113,9 @@
 
     def test_node_references_are_cr_delimited(self):
         builder = GraphIndexBuilder(reference_lists=1)
-        builder.add_node('reference', ([], ), 'data')
-        builder.add_node('reference2', ([], ), 'data')
-        builder.add_node('key', (['reference', 'reference2'], ), 'data')
+        builder.add_node('reference', 'data', ([], ))
+        builder.add_node('reference2', 'data', ([], ))
+        builder.add_node('key', 'data', (['reference', 'reference2'], ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=1\n"
@@ -118,8 +126,8 @@
 
     def test_multiple_reference_lists_are_tab_delimited(self):
         builder = GraphIndexBuilder(reference_lists=2)
-        builder.add_node('keference', ([], []), 'data')
-        builder.add_node('rey', (['keference'], ['keference']), 'data')
+        builder.add_node('keference', 'data', ([], []))
+        builder.add_node('rey', 'data', (['keference'], ['keference']))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=2\n"
@@ -129,7 +137,7 @@
 
     def test_add_node_referencing_missing_key_makes_absent(self):
         builder = GraphIndexBuilder(reference_lists=1)
-        builder.add_node('rey', (['beference', 'aeference2'], ), 'data')
+        builder.add_node('rey', 'data', (['beference', 'aeference2'], ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=1\n"
@@ -142,7 +150,7 @@
         # test the node digit expands as needed.
         builder = GraphIndexBuilder(reference_lists=1)
         references = map(str, reversed(range(9)))
-        builder.add_node('2-key', (references, ), '')
+        builder.add_node('2-key', '', (references, ))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=1\n"
@@ -162,7 +170,7 @@
         # the offsets after an absent record should be correct when there are
         # >1 reference lists.
         builder = GraphIndexBuilder(reference_lists=2)
-        builder.add_node('parent', (['aail', 'zther'], []), '')
+        builder.add_node('parent', '', (['aail', 'zther'], []))
         stream = builder.finish()
         contents = stream.read()
         self.assertEqual("Bazaar Graph Index 1\nnode_ref_lists=2\n"
@@ -175,66 +183,68 @@
         builder = GraphIndexBuilder()
         for bad_char in '\t\n\x0b\x0c\r\x00 ':
             self.assertRaises(errors.BadIndexKey, builder.add_node,
-                'a%skey' % bad_char, (), 'data')
+                'a%skey' % bad_char, 'data')
         self.assertRaises(errors.BadIndexKey, builder.add_node,
-                '', (), 'data')
+                '', 'data')
 
     def test_add_node_bad_data(self):
         builder = GraphIndexBuilder()
         self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            (), 'data\naa')
+            'data\naa')
         self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            (), 'data\x00aa')
+            'data\x00aa')
 
     def test_add_node_bad_mismatched_ref_lists_length(self):
         builder = GraphIndexBuilder()
         self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            ([], ), 'data aa')
+            'data aa', ([], ))
         builder = GraphIndexBuilder(reference_lists=1)
         self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            (), 'data aa')
-        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            ([], []), 'data aa')
+            'data aa')
+        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
+            'data aa', (), )
+        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
+            'data aa', ([], []))
         builder = GraphIndexBuilder(reference_lists=2)
         self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            (), 'data aa')
-        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            ([], ), 'data aa')
-        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
-            ([], [], []), 'data aa')
+            'data aa')
+        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
+            'data aa', ([], ))
+        self.assertRaises(errors.BadIndexValue, builder.add_node, 'akey',
+            'data aa', ([], [], []))
 
     def test_add_node_bad_key_in_reference_lists(self):
         # first list, first key - trivial
         builder = GraphIndexBuilder(reference_lists=1)
         self.assertRaises(errors.BadIndexKey, builder.add_node, 'akey',
-            (['a key'], ), 'data aa')
+            'data aa', (['a key'], ))
         # need to check more than the first key in the list
         self.assertRaises(errors.BadIndexKey, builder.add_node, 'akey',
-            (['agoodkey', 'this is a bad key'], ), 'data aa')
+            'data aa', (['agoodkey', 'this is a bad key'], ))
         # and if there is more than one list it should be getting checked
         # too
         builder = GraphIndexBuilder(reference_lists=2)
         self.assertRaises(errors.BadIndexKey, builder.add_node, 'akey',
-            ([], ['a bad key']), 'data aa')
+            'data aa', ([], ['a bad key']))
 
     def test_add_duplicate_key(self):
         builder = GraphIndexBuilder()
-        builder.add_node('key', (), 'data')
+        builder.add_node('key', 'data')
         self.assertRaises(errors.BadIndexDuplicateKey, builder.add_node, 'key',
-            (), 'data')
+            'data')
 
     def test_add_key_after_referencing_key(self):
         builder = GraphIndexBuilder(reference_lists=1)
-        builder.add_node('key', (['reference'], ), 'data')
-        builder.add_node('reference', ([],), 'data')
+        builder.add_node('key', 'data', (['reference'], ))
+        builder.add_node('reference', 'data', ([],))
 
 
 class TestGraphIndex(TestCaseWithMemoryTransport):
 
     def make_index(self, ref_lists=0, nodes=[]):
         builder = GraphIndexBuilder(ref_lists)
-        for node, references, value in nodes:
-            builder.add_node(node, references, value)
+        for node, value, references in nodes:
+            builder.add_node(node, value, references)
         stream = builder.finish()
         trans = self.get_transport()
         trans.put_file('index', stream)
@@ -250,33 +260,33 @@
         self.assertEqual([], list(index.iter_all_entries()))
 
     def test_iter_all_entries_simple(self):
-        index = self.make_index(nodes=[('name', (), 'data')])
-        self.assertEqual([('name', (), 'data')],
+        index = self.make_index(nodes=[('name', 'data', ())])
+        self.assertEqual([('name', 'data')],
             list(index.iter_all_entries()))
 
     def test_iter_all_entries_references_resolved(self):
         index = self.make_index(1, nodes=[
-            ('name', (['ref'], ), 'data'),
-            ('ref', ([], ), 'refdata')])
-        self.assertEqual(set([('name', (('ref',),), 'data'),
-            ('ref', ((), ), 'refdata')]),
+            ('name', 'data', (['ref'], )),
+            ('ref', 'refdata', ([], ))])
+        self.assertEqual(set([('name', 'data', (('ref',),)),
+            ('ref', 'refdata', ((), ))]),
             set(index.iter_all_entries()))
 
     def test_iteration_absent_skipped(self):
         index = self.make_index(1, nodes=[
-            ('name', (['ref'], ), 'data')])
-        self.assertEqual(set([('name', (('ref',),), 'data')]),
+            ('name', 'data', (['ref'], ))])
+        self.assertEqual(set([('name', 'data', (('ref',),))]),
             set(index.iter_all_entries()))
-        self.assertEqual(set([('name', (('ref',),), 'data')]),
+        self.assertEqual(set([('name', 'data', (('ref',),))]),
             set(index.iter_entries(['name'])))
         self.assertEqual([], list(index.iter_entries(['ref'])))
 
     def test_iter_all_keys(self):
         index = self.make_index(1, nodes=[
-            ('name', (['ref'], ), 'data'),
-            ('ref', ([], ), 'refdata')])
-        self.assertEqual(set([('name', (('ref',),), 'data'),
-            ('ref', ((), ), 'refdata')]),
+            ('name', 'data', (['ref'], )),
+            ('ref', 'refdata', ([], ))])
+        self.assertEqual(set([('name', 'data', (('ref',),)),
+            ('ref', 'refdata', ((), ))]),
             set(index.iter_entries(['name', 'ref'])))
 
     def test_iter_nothing_empty(self):
@@ -311,7 +321,7 @@
         self.assertRaises(errors.BadIndexData, index.validate)
 
     def test_validate_missing_end_line_nonempty(self):
-        index = self.make_index(2, [('key', ([], []), '')])
+        index = self.make_index(2, [('key', '', ([], []))])
         trans = self.get_transport()
         content = trans.get_bytes('index')
         # truncate the last byte
@@ -323,7 +333,7 @@
         index.validate()
 
     def test_validate_no_refs_content(self):
-        index = self.make_index(nodes=[('key', (), 'value')])
+        index = self.make_index(nodes=[('key', 'value', ())])
         index.validate()
 
 
@@ -331,8 +341,8 @@
 
     def make_index(self, name, ref_lists=0, nodes=[]):
         builder = GraphIndexBuilder(ref_lists)
-        for node, references, value in nodes:
-            builder.add_node(node, references, value)
+        for node, value, references in nodes:
+            builder.add_node(node, value, references)
         stream = builder.finish()
         trans = self.get_transport()
         trans.put_file(name, stream)
@@ -345,9 +355,9 @@
 
     def test_add_index(self):
         index = CombinedGraphIndex([])
-        index1 = self.make_index('name', nodes=[('key', (), '')])
+        index1 = self.make_index('name', 0, nodes=[('key', '', ())])
         index.insert_index(0, index1)
-        self.assertEqual([('key', (), '')], list(index.iter_all_entries()))
+        self.assertEqual([('key', '')], list(index.iter_all_entries()))
 
     def test_iter_all_entries_empty(self):
         index = CombinedGraphIndex([])
@@ -359,31 +369,31 @@
         self.assertEqual([], list(index.iter_all_entries()))
 
     def test_iter_all_entries_simple(self):
-        index1 = self.make_index('name', nodes=[('name', (), 'data')])
+        index1 = self.make_index('name', nodes=[('name', 'data', ())])
         index = CombinedGraphIndex([index1])
-        self.assertEqual([('name', (), 'data')],
+        self.assertEqual([('name', 'data')],
             list(index.iter_all_entries()))
 
     def test_iter_all_entries_two_indices(self):
-        index1 = self.make_index('name1', nodes=[('name', (), 'data')])
-        index2 = self.make_index('name2', nodes=[('2', (), '')])
+        index1 = self.make_index('name1', nodes=[('name', 'data', ())])
+        index2 = self.make_index('name2', nodes=[('2', '', ())])
         index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([('name', (), 'data'),
-            ('2', (), '')],
+        self.assertEqual([('name', 'data'),
+            ('2', '')],
             list(index.iter_all_entries()))
 
     def test_iter_entries_two_indices_dup_key(self):
-        index1 = self.make_index('name1', nodes=[('name', (), 'data')])
-        index2 = self.make_index('name2', nodes=[('name', (), 'data')])
+        index1 = self.make_index('name1', nodes=[('name', 'data', ())])
+        index2 = self.make_index('name2', nodes=[('name', 'data', ())])
         index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([('name', (), 'data')],
+        self.assertEqual([('name', 'data')],
             list(index.iter_entries(['name'])))
 
     def test_iter_all_entries_two_indices_dup_key(self):
-        index1 = self.make_index('name1', nodes=[('name', (), 'data')])
-        index2 = self.make_index('name2', nodes=[('name', (), 'data')])
+        index1 = self.make_index('name1', nodes=[('name', 'data', ())])
+        index2 = self.make_index('name2', nodes=[('name', 'data', ())])
         index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([('name', (), 'data')],
+        self.assertEqual([('name', 'data')],
             list(index.iter_all_entries()))
 
     def test_iter_nothing_empty(self):
@@ -397,23 +407,23 @@
 
     def test_iter_all_keys(self):
         index1 = self.make_index('1', 1, nodes=[
-            ('name', (['ref'], ), 'data')])
+            ('name', 'data', (['ref'], ))])
         index2 = self.make_index('2', 1, nodes=[
-            ('ref', ([], ), 'refdata')])
+            ('ref', 'refdata', ((), ))])
         index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(set([('name', (('ref',),), 'data'),
-            ('ref', ((), ), 'refdata')]),
+        self.assertEqual(set([('name', 'data', (('ref', ), )),
+            ('ref', 'refdata', ((), ))]),
             set(index.iter_entries(['name', 'ref'])))
  
     def test_iter_all_keys_dup_entry(self):
         index1 = self.make_index('1', 1, nodes=[
-            ('name', (['ref'], ), 'data'),
-            ('ref', ([], ), 'refdata')])
+            ('name', 'data', (['ref'], )),
+            ('ref', 'refdata', ([], ))])
         index2 = self.make_index('2', 1, nodes=[
-            ('ref', ([], ), 'refdata')])
+            ('ref', 'refdata', ([], ))])
         index = CombinedGraphIndex([index1, index2])
-        self.assertEqual(set([('name', (('ref',),), 'data'),
-            ('ref', ((), ), 'refdata')]),
+        self.assertEqual(set([('name', 'data', (('ref',),)),
+            ('ref', 'refdata', ((), ))]),
             set(index.iter_entries(['name', 'ref'])))
  
     def test_iter_missing_entry_empty(self):
@@ -432,14 +442,14 @@
         self.assertEqual([], list(index.iter_entries(['a'])))
  
     def test_iter_entry_present_one_index_only(self):
-        index1 = self.make_index('1', nodes=[('key', (), '')])
+        index1 = self.make_index('1', nodes=[('key', '', ())])
         index2 = self.make_index('2', nodes=[])
         index = CombinedGraphIndex([index1, index2])
-        self.assertEqual([('key', (), '')],
+        self.assertEqual([('key', '')],
             list(index.iter_entries(['key'])))
         # and in the other direction
         index = CombinedGraphIndex([index2, index1])
-        self.assertEqual([('key', (), '')],
+        self.assertEqual([('key', '')],
             list(index.iter_entries(['key'])))
 
     def test_validate_bad_child_index_errors(self):
@@ -463,12 +473,12 @@
 
     def test_add_nodes(self):
         index = self.make_index(1)
-        index.add_nodes([('name', ([],), 'data')])
-        index.add_nodes([('name2', ([],), ''), ('name3', (['r'],), '')])
+        index.add_nodes([('name', 'data', ([],))])
+        index.add_nodes([('name2', '', ([],)), ('name3', '', (['r'],))])
         self.assertEqual(set([
-            ('name', ((),), 'data'),
-            ('name2', ((),), ''),
-            ('name3', (('r',),), ''),
+            ('name', 'data', ((),)),
+            ('name2', '', ((),)),
+            ('name3', '', (('r',),)),
             ]), set(index.iter_all_entries()))
 
     def test_iter_all_entries_empty(self):
@@ -476,33 +486,33 @@
         self.assertEqual([], list(index.iter_all_entries()))
 
     def test_iter_all_entries_simple(self):
-        index = self.make_index(nodes=[('name', (), 'data')])
-        self.assertEqual([('name', (), 'data')],
+        index = self.make_index(nodes=[('name', 'data', ())])
+        self.assertEqual([('name', 'data')],
             list(index.iter_all_entries()))
 
     def test_iter_all_entries_references(self):
         index = self.make_index(1, nodes=[
-            ('name', (['ref'], ), 'data'),
-            ('ref', ([], ), 'refdata')])
-        self.assertEqual(set([('name', (('ref',),), 'data'),
-            ('ref', ((), ), 'refdata')]),
+            ('name', 'data', (['ref'], )),
+            ('ref', 'refdata', ([], ))])
+        self.assertEqual(set([('name', 'data', (('ref',),)),
+            ('ref', 'refdata', ((), ))]),
             set(index.iter_all_entries()))
 
     def test_iteration_absent_skipped(self):
         index = self.make_index(1, nodes=[
-            ('name', (['ref'], ), 'data')])
-        self.assertEqual(set([('name', (('ref',),), 'data')]),
+            ('name', 'data', (['ref'], ))])
+        self.assertEqual(set([('name', 'data', (('ref',),))]),
             set(index.iter_all_entries()))
-        self.assertEqual(set([('name', (('ref',),), 'data')]),
+        self.assertEqual(set([('name', 'data', (('ref',),))]),
             set(index.iter_entries(['name'])))
         self.assertEqual([], list(index.iter_entries(['ref'])))
 
     def test_iter_all_keys(self):
         index = self.make_index(1, nodes=[
-            ('name', (['ref'], ), 'data'),
-            ('ref', ([], ), 'refdata')])
-        self.assertEqual(set([('name', (('ref',),), 'data'),
-            ('ref', ((), ), 'refdata')]),
+            ('name', 'data', (['ref'], )),
+            ('ref', 'refdata', ([], ))])
+        self.assertEqual(set([('name', 'data', (('ref',),)),
+            ('ref', 'refdata', ((), ))]),
             set(index.iter_entries(['name', 'ref'])))
 
     def test_iter_nothing_empty(self):
@@ -518,7 +528,7 @@
         index.validate()
 
     def test_validate_no_refs_content(self):
-        index = self.make_index(nodes=[('key', (), 'value')])
+        index = self.make_index(nodes=[('key', 'value', ())])
         index.validate()
 
 

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2007-07-15 08:30:05 +0000
+++ bzrlib/tests/test_knit.py	2007-07-15 15:52:54 +0000
@@ -941,8 +941,8 @@
         self.assertEqualDiff(''.join(k.get_lines('text-1a')), TEXT_1A)
         # check the index had the right data added.
         self.assertEqual(set([
-            ('text-1', ((), ()), ' 0 127'),
-            ('text-1a', (('text-1',), ('text-1',)), ' 127 140'),
+            ('text-1', ' 0 127', ((), ())),
+            ('text-1a', ' 127 140', (('text-1',), ('text-1',))),
             ]), set(index.iter_all_entries()))
         # we should not have a .kndx file
         self.assertFalse(get_transport('.').has('test.kndx'))
@@ -1546,18 +1546,18 @@
         # build a complex graph across several indices.
         if deltas:
             index1 = self.make_g_index('1', 2, [
-                ('tip', (['parent'], [], ), 'N0 100'),
-                ('tail', ([], []), '')])
+                ('tip', 'N0 100', (['parent'], [], )),
+                ('tail', '', ([], []))])
             index2 = self.make_g_index('2', 2, [
-                ('parent', (['tail', 'ghost'], ['tail']), ' 100 78'),
-                ('separate', ([], []), '')])
+                ('parent', ' 100 78', (['tail', 'ghost'], ['tail'])),
+                ('separate', '', ([], []))])
         else:
             index1 = self.make_g_index('1', 1, [
-                ('tip', (['parent'], ), 'N0 100'),
-                ('tail', ([], ), '')])
+                ('tip', 'N0 100', (['parent'], )),
+                ('tail', '', ([], ))])
             index2 = self.make_g_index('2', 1, [
-                ('parent', (['tail', 'ghost'], ), ' 100 78'),
-                ('separate', ([], ), '')])
+                ('parent', ' 100 78', (['tail', 'ghost'], )),
+                ('separate', '', ([], ))])
         combined_index = CombinedGraphIndex([index1, index2])
         if catch_adds:
             self.combined_index = combined_index
@@ -1706,7 +1706,7 @@
     def test_add_version_smoke(self):
         index = self.two_graph_index(catch_adds=True)
         index.add_version('new', 'fulltext,no-eol', 50, 60, ['separate'])
-        self.assertEqual([[('new', (('separate',),), 'N50 60')]],
+        self.assertEqual([[('new', 'N50 60', (('separate',),))]],
             self.caught_entries)
 
     def test_add_version_delta_not_delta_index(self):
@@ -1748,8 +1748,8 @@
                 ('new', 'fulltext,no-eol', 50, 60, ['separate']),
                 ('new2', 'fulltext', 0, 6, ['new']),
                 ])
-        self.assertEqual([('new', (('separate',),), 'N50 60'),
-            ('new2', (('new',),), ' 0 6')],
+        self.assertEqual([('new', 'N50 60', (('separate',),)),
+            ('new2', ' 0 6', (('new',),))],
             sorted(self.caught_entries[0]))
         self.assertEqual(1, len(self.caught_entries))
 
@@ -1759,8 +1759,8 @@
                 ('new', 'fulltext,no-eol', 50, 60, ['separate']),
                 ('new2', 'line-delta', 0, 6, ['new']),
                 ])
-        self.assertEqual([('new', (('separate',), ()), 'N50 60'),
-            ('new2', (('new',), ('new',), ), ' 0 6')],
+        self.assertEqual([('new', 'N50 60', (('separate',), ())),
+            ('new2', ' 0 6', (('new',), ('new',), ))],
             sorted(self.caught_entries[0]))
         self.assertEqual(1, len(self.caught_entries))
 

=== modified file 'doc/developers/indices.txt'
--- doc/developers/indices.txt	2007-07-15 07:31:37 +0000
+++ doc/developers/indices.txt	2007-07-15 15:40:37 +0000
@@ -49,9 +49,9 @@
 signatures to operate would strive to keep the signatures and return
 values as similar as possible. e.g.::
 
-    GraphIndexBuilder - add_node(key, references, value)
+    GraphIndexBuilder - add_node(key, value, references)
     IndexBuilder - add_node(key, value)
-    WhackyIndexBuilder - add_node(key, whackiness, value)
+    WhackyIndexBuilder - add_node(key, value, whackiness)
 
 as opposed to something quite different like::
 

